﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;


namespace CSharpTCIO_Example
{
  public partial class Form_main : Form
  {
    #region members
    const string _tcioLibFileName = "tcio.dll";
    const Int32 DLL_SUCCESS = 0;
    const Int32 DLL_FAIL = -1;
    Int32 comPortStart = 1, comPortEnd = 10, comPortFound = 0;
    Int32 dllState = 0, dllError = 0;
    Boolean connected;
    UInt32 controlIn;
    Double p_vinc, p_iinc, p_pinc, p_rinc, vsinc, isinc, psinc, rsinc;
    Double voltagePhysMax, currentPhysMax, powerPhysMax, resistancePhysMax;
    Double voltagePhysMin, currentPhysMin, powerPhysMin, resistancePhysMin;
    Boolean loadRefValues;
    Boolean isVoltageRefValuePresent;
    Boolean isCurrentRefValuePresent;
    Boolean isPowerRefValuePresent;

    Color changedColor = Color.Yellow;
    Color presentStateColor;

    #endregion

    #region tcio dll imports
    //init
    [DllImport(_tcioLibFileName, EntryPoint = "DllInit", CallingConvention = CallingConvention.Cdecl)]
    public static extern int DllInit();
    [DllImport(_tcioLibFileName, EntryPoint = "DllClose", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 DllClose();
    [DllImport(_tcioLibFileName, EntryPoint = "DllGetStatus", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 DllGetStatus(ref Int32 p_dllState, ref Int32 p_dllError);
    [DllImport(_tcioLibFileName, EntryPoint = "DllReadVersion", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 DllReadVersion(ref UInt32 p_version, ref UInt32 p_build, StringBuilder p_string);

    [DllImport(_tcioLibFileName, EntryPoint = "DllSearchDevice", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 DllSearchDevice(Int32 fromport, Int32 toport, ref Int32 p_portnrfound);

    //mandatory
    [DllImport(_tcioLibFileName, EntryPoint = "TC4GetPhysicalValuesIncrement", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4GetPhysicalValuesIncrement( ref Double p_vinc, ref Double p_iinc,
                                                              ref Double p_pinc, ref Double p_rinc,
                                                              ref Double v_sinc, ref Double isinc,
                                                              ref Double p_sinc, ref Double rsinc);

    [DllImport(_tcioLibFileName, EntryPoint = "TC4SetModuleSelector", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4SetModuleSelector(UInt32 selector);

    // actual values getters
    [DllImport(_tcioLibFileName, EntryPoint = "TC4GetSystemPhysicalLimitMax", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4GetSystemPhysicalLimitMax(ref Double voltagePhysMax, ref Double  currentPhysMax,
                                                            ref Double  powerPhysMax,  ref Double  resistancePhysMax);
    [DllImport(_tcioLibFileName, EntryPoint = "TC4GetSystemPhysicalLimitMin", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4GetSystemPhysicalLimitMin(ref Double voltagePhysMin, ref Double currentPhysMin,
                                                            ref Double powerPhysMin, ref Double resistancePhysMin);
    [DllImport(_tcioLibFileName, EntryPoint = "TC4GetVoltageAct", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4GetVoltageAct(ref Double p_Voltage);
    [DllImport(_tcioLibFileName, EntryPoint = "TC4GetCurrentAct", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4GetCurrentAct(ref Double p_Current);
    [DllImport(_tcioLibFileName, EntryPoint = "TC4GetPowerAct", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4GetPowerAct(ref Double p_Power);

    // reference values getters and setters
    [DllImport(_tcioLibFileName, EntryPoint = "TC4GetVoltageRef", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4GetVoltageRef(ref Double p_Voltage);
    [DllImport(_tcioLibFileName, EntryPoint = "TC4GetCurrentRef", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4GetCurrentRef(ref Double p_Current);
    [DllImport(_tcioLibFileName, EntryPoint = "TC4GetPowerRef", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4GetPowerRef(ref Double p_Power);

    [DllImport(_tcioLibFileName, EntryPoint = "TC4SetVoltageRef", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4SetVoltageRef(Double Voltage);
    [DllImport(_tcioLibFileName, EntryPoint = "TC4SetCurrentRef", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4SetCurrentRef(Double Current);
    [DllImport(_tcioLibFileName, EntryPoint = "TC4SetPowerRef", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4SetPowerRef(Double Power);

    [DllImport(_tcioLibFileName, EntryPoint = "TC4SetRemoteControlInput", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4SetRemoteControlInput(UInt32 control);

    [DllImport(_tcioLibFileName, EntryPoint = "TC4GetControlIn", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4GetControlIn(ref UInt32 p_control);
    [DllImport(_tcioLibFileName, EntryPoint = "TC4SetControlIn", CallingConvention = CallingConvention.Cdecl)]
    public static extern Int32 TC4SetControlIn(UInt32 control);

    #endregion

    public Form_main()
    {
      InitializeComponent();
      presentStateColor = textBox_voltageRef.BackColor;
      UInt32 version = 0;
      UInt32 build = 0;
      StringBuilder sb = new StringBuilder(256);
      DllReadVersion(ref version, ref build, sb);
      toolStripStatusLabel_tcioText.Text = "TCIO V" + (version >> 16).ToString() +
                                           "." + (version & 0xFFFF).ToString() + 
                                           " " + sb.ToString();
    }

    private void button_powerOn_Click(object sender, EventArgs e)
    {
      TC4SetControlIn(1);
    }

    private void button_powerOff_Click(object sender, EventArgs e)
    {
      TC4SetControlIn(0);
    }

    private void buttont_remoteControl_Click(object sender, EventArgs e)
    {
      TC4SetRemoteControlInput(2);
    }

    private void button_connect_Click(object sender, EventArgs e)
    {
      try
      {
        string[] comPorts = textBox_comPort.Text.Split(new Char[] {' ', ',', '-', ':', '/' });
        int i = 0;
        foreach (string s in comPorts)
        {
          if (s.Trim() != "")
          { comPorts[i++] = s; }
        }

        if (comPorts.Length > 0)
        { comPortStart = Convert.ToInt32(comPorts[0]); }
        if (comPorts.Length > 1)
        { comPortEnd = Convert.ToInt32(comPorts[1]); }
        else
        { comPortEnd = comPortStart; }
      }catch (Exception)
      {
        MessageBox.Show("Failed to parse COM Port(s).", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        return;
      }
      
      Int32 result = DllSearchDevice(comPortStart, comPortEnd, ref comPortFound);
      if (0 == comPortFound)
      {
        MessageBox.Show("No TopCon connected", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
        return;
      }
      result = TC4GetPhysicalValuesIncrement(ref p_vinc, ref p_iinc, ref p_pinc, ref p_rinc, ref vsinc, ref isinc, ref psinc, ref rsinc);
      result = TC4GetSystemPhysicalLimitMax(ref voltagePhysMax, ref currentPhysMax, ref powerPhysMax, ref resistancePhysMax);
      result = TC4GetSystemPhysicalLimitMin(ref voltagePhysMin, ref currentPhysMin, ref powerPhysMin, ref resistancePhysMin);
      result = TC4SetModuleSelector(64);
      result = DllGetStatus(ref dllState, ref dllError);
      if ((result != 0) || (dllState != 0))
      {
        MessageBox.Show("DllInit failed", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
      else
      {
        MessageBox.Show("TopCon connected to port: " + comPortFound.ToString(), "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
        textBox_comPort.Text = comPortFound.ToString();
        connected = true;
        //get init values
        ReadVoltageRef();
        ReadCurrentRef();
        ReadPowerRef();
        result = DllGetStatus(ref dllState, ref dllError);
        if ((result != 0) || (dllState != 0))
        {
          MessageBox.Show("Error ocured while loading data.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        Timer_getValues.Enabled = true;
      }
    }

    private void button_disconnect_Click(object sender, EventArgs e)
    {
      Int32 result = DllClose();
      if (result != DLL_SUCCESS)
      { MessageBox.Show("TopCon not disconnected", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error); }
      else
      { connected = false; }
    }

    private void Form_main_Load(object sender, EventArgs e)
    {
      Int32 result = DllInit();
      if (result != DLL_SUCCESS)
      { MessageBox.Show("DllInit failed", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error); }
      connected = false;
    }

    private void Timer_getValues_Tick(object sender, EventArgs e)
    {
      Int32 result;
      Double voltageAct = 0.0;
      Double currentAct = 0.0;
      Double powerAct = 0.0;
      if (connected)
      {
        //get values from device
        result = TC4GetVoltageAct(ref voltageAct);
        result = TC4GetCurrentAct(ref currentAct);
        result = TC4GetPowerAct(ref powerAct);
        label_voltageAct.Text = String.Format("{0:0.00}", voltageAct);
        label_currentAct.Text = String.Format("{0:0.00}", currentAct);
        label_powerAct.Text = String.Format("{0:0.00}", powerAct);
        result = TC4GetControlIn(ref controlIn);
        if (controlIn != 0)
        { label_controlState.BackColor = Color.Green; }
        else
        { label_controlState.BackColor = Color.Red; }

        result = DllGetStatus(ref dllState, ref dllError);
        if ((result != 0) || (dllState != 0))
        {
          SetDisconectState();
          MessageBox.Show("Error ocured while loading data.\nTopCon disconnected.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
      }
      else
      {
        SetDisconectState();
        MessageBox.Show("TopCon disconnected", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
      }
    }

    private void SetDisconectState()
    {
      Timer_getValues.Enabled = false;
      connected = false;
      label_voltageAct.Text = "0";
      label_currentAct.Text = "0";
      label_powerAct.Text = "0";
      loadRefValues = true;
      textBox_voltageRef.Text = "0";
      textBox_currentRef.Text = "0";
      textBox_powerRef.Text = "0";
      loadRefValues = false;
      label_controlState.BackColor = Color.Gray;
    }

    private void Form_main_FormClosed(object sender, FormClosedEventArgs e)
    {
      Timer_getValues.Enabled = false;
      connected = false;
      DllClose();
    }

    private void textBox_voltageRef_TextChanged(object sender, EventArgs e)
    {
      if (!connected || loadRefValues) { return; }
      isVoltageRefValuePresent = false;
      textBox_voltageRef.BackColor = changedColor;
    }

    private void textBox_currentRef_TextChanged(object sender, EventArgs e)
    {
      if (!connected || loadRefValues) { return; }
      isCurrentRefValuePresent = false;
      textBox_currentRef.BackColor = changedColor;
    }

    private void textBox_powerRef_TextChanged(object sender, EventArgs e)
    {
      if (!connected || loadRefValues) { return; }
      isPowerRefValuePresent = false;
      textBox_powerRef.BackColor = changedColor;
    }

    private void textBox_voltageRef_KeyDown(object sender, KeyEventArgs e)
    {
      if ( (e.KeyCode == Keys.Enter) && !isVoltageRefValuePresent && connected)
      { WriteVoltageRef(); }
    }

    private void textBox_currentRef_KeyDown(object sender, KeyEventArgs e)
    {
      if ( (e.KeyCode == Keys.Enter) && !isCurrentRefValuePresent && connected)
      { WriteCurrentRef(); }
    }

    private void textBox_powerRef_KeyDown(object sender, KeyEventArgs e)
    {
      if ( (e.KeyCode == Keys.Enter) && !isPowerRefValuePresent && connected)
      { WritePowerRef(); }
    }

    private void textBox_voltageRef_Leave(object sender, EventArgs e)
    {
      if (!isVoltageRefValuePresent && connected)
      { WriteVoltageRef(); }
    }

    private void textBox_currentRef_Leave(object sender, EventArgs e)
    {
      if (!isCurrentRefValuePresent && connected)
      { WriteCurrentRef(); }
    }

    private void textBox_powerRef_Leave(object sender, EventArgs e)
    {
      if (!isPowerRefValuePresent && connected)
      { WritePowerRef(); }
    }

    private Int32 ReadVoltageRef()
    {
      Int32 result = DLL_FAIL;
      Double voltageRef = 0.0;
      result = TC4GetVoltageRef(ref voltageRef);
      loadRefValues = true;
      try
      {
        if (DLL_SUCCESS == result)
        {
          textBox_voltageRef.Text = String.Format("{0:0.00}", voltageRef);
          isCurrentRefValuePresent = true;
        }
      }
      catch (Exception) { }

      loadRefValues = false;
      return result;
    }

    private Int32 ReadCurrentRef()
    {
      Int32 result = DLL_FAIL;
      Double currentRef = 0.0;
      result = TC4GetCurrentRef(ref currentRef);
      loadRefValues = true;
      try
      {
        if (DLL_SUCCESS == result)
        {
          textBox_currentRef.Text = String.Format("{0:0.00}", currentRef);
          isCurrentRefValuePresent = true;
        }
      }
      catch (Exception) { }

      loadRefValues = false;
      return result;
    }

    private Int32 ReadPowerRef()
    {
      Int32 result = DLL_FAIL;
      Double powerRef = 0.0;
      result = TC4GetPowerRef(ref powerRef);
      loadRefValues = true;
      try
      {
        if (DLL_SUCCESS == result )
        {
          textBox_powerRef.Text = String.Format("{0:0.00}", powerRef);
          isPowerRefValuePresent = true;
        }
      }
      catch (Exception) { }
      loadRefValues = false;
      return result;
    }

    private Int32 WriteVoltageRef()
    {
      Int32 result = DLL_FAIL;
      try
      {
        Double voltageRef = Convert.ToDouble(textBox_voltageRef.Text);
        voltageRef = GetValueInnerLimits(voltageRef, voltagePhysMin, voltagePhysMax);
        result = TC4SetVoltageRef(voltageRef);
        if (DLL_SUCCESS == result)
        {
          result = ReadVoltageRef();
          isVoltageRefValuePresent = true;
          textBox_voltageRef.BackColor = presentStateColor;
        }
      }catch(Exception) { }
      return result;
    }

    private Int32 WriteCurrentRef()
    {
      Int32 result = DLL_FAIL;
      try
      {
        Double currentRef = Convert.ToDouble(textBox_currentRef.Text);
        currentRef = GetValueInnerLimits(currentRef, currentPhysMin, currentPhysMax);
        result = TC4SetCurrentRef(currentRef);
        if (DLL_SUCCESS == result)
        {
          result = ReadCurrentRef();
          isCurrentRefValuePresent = true;
          textBox_currentRef.BackColor = presentStateColor;
        }
      }
      catch (Exception) { }
      return result;
    }

    private Int32 WritePowerRef()
    {
      Int32 result = DLL_FAIL;
      try
      {
        Double powerRef = Convert.ToDouble(textBox_powerRef.Text);
        powerRef = GetValueInnerLimits(powerRef, powerPhysMin, powerPhysMax);
        result = TC4SetPowerRef(powerRef);
        if (DLL_SUCCESS == result)
        {
          result = ReadPowerRef();
          isPowerRefValuePresent = true;
          textBox_powerRef.BackColor = presentStateColor;
        }
      }
      catch (Exception) { }
      return result;
    }

    private Double GetValueInnerLimits(Double value, Double min, Double max)
    {
      if (value > max)
      {value = max; }
      if (value < min)
      { value = min; }
      return value;
    }
  }
}
