#include "stdafx.h"

#include "helper.h"

#include <windows.h>

#include <cstdio>
#include <sstream>
#include <iostream>
#include <limits>
#include <string>
#include <ctime>

#include "serialiolib.h"
#include "profiler.h"

using namespace std;

void PressEnterToContinue()
{
  std::cout << "Press ENTER to continue... " << flush;
  std::cin.ignore( std::numeric_limits <std::streamsize> ::max(), '\n' );
}

int WaitForFlashActionResult(void)
{
  int miliSecondsToWait = 5000;
  const int actionWaitTime = 10;
  bool busy = true;
  int actionResult;

  while (busy)
  {
    if (DLL_SUCCESS != TC4GetFnSeqActionResult(&actionResult))
    {
      cout << "result TC4GetFnSeqActionResult: " << DLL_FAIL << endl;
      return DLL_FAIL;
    }
    if (actionResult == 0)
    {
      busy = false;
    }
    else if (actionResult < 0)
    {
      cout << "actionResult TC4GetFnSeqActionResult: " << actionResult << endl;
      return DLL_FAIL;
    }
    else
    {
      Sleep(actionWaitTime);
      if (miliSecondsToWait > 0)
      {
        cout << "TC4GetFnSeqActionResult Busy" << endl;
        miliSecondsToWait = miliSecondsToWait - actionWaitTime;
      }
      else
      {
        cout << "TC4GetFnSeqActionResult Timeout!" << endl;
        return DLL_FAIL;
      }
    }
  }

  return DLL_SUCCESS;
}
DLL_RET_T FuncgenCreateLookupFactors(void)
{
  T_FnSeqHeader Header;
  T_FnSeqHeaderExtension HeaderExtension;
  T_SASTechnologyParamsEN50530 TechnologyParams;

  if (DLL_SUCCESS != TC4GetFnSeqHeader(&Header))
  {
    cout << "result TC4GetFnSeqHeader: " << DLL_FAIL << endl;
    return DLL_FAIL;
  }

  if (Header.HeaderExtensionVersion != 2)
  {
    cout << "Header.HeaderExtensionVersion != 2 : " << DLL_FAIL << endl;
    return DLL_FAIL;
  }

  if (DLL_SUCCESS != TC4GetFnSeqHeaderExtension(Header.HeaderExtensionVersion, &HeaderExtension, &TechnologyParams))
  {
    cout << "result TC4GetFnSeqHeaderExtension: " << DLL_FAIL << endl;
    return DLL_FAIL;
  }

  int error = 0;
  if (DLL_SUCCESS != TC4CreateSASIrradianceAndTemperatureLookup(&TechnologyParams, Header.HeaderExtensionVersion, &HeaderExtension, &error))
  {
    cout << "result TC4CreateSASIrradianceAndTemperatureLookup: " << DLL_FAIL << endl;
    return DLL_FAIL;
  }

  return DLL_SUCCESS;
}

DLL_RET_T LoadFuncGenSAS(unsigned int seqNr)
{
  //normal funcgen load
  if (DLL_SUCCESS != TC4SetFnSeqActionLoad(seqNr))
  {
    cout << "result TC4SetFnSeqActionLoad: " << DLL_FAIL << endl;
    return DLL_FAIL;
  }

  if (DLL_SUCCESS != WaitForFlashActionResult())
  {
    cout << "result WaitForFlashActionResult: " << DLL_FAIL << endl;
    return DLL_FAIL;
  }

  if (DLL_SUCCESS != TC4SetSASFuncgenInterfaceEnabled())
  {
    cout << "result TC4SetSASFuncgenInterfaceEnabled: " << DLL_FAIL << endl;
    return DLL_FAIL;
  }

  //now the sas fungen stuff
  T_FnSeqHeader Header;
  T_FnSeqHeaderExtension HeaderExtension;
  T_SASTechnologyParamsEN50530 TechnologyParams;

  if (DLL_SUCCESS != TC4GetFnSeqHeader(&Header))
  {
    cout << "result TC4GetFnSeqHeader: " << DLL_FAIL << endl;
    return DLL_FAIL;
  }

  if (Header.HeaderExtensionVersion != 2)
  {
    cout << "Header.HeaderExtensionVersion != 2 : " << DLL_FAIL << endl;
    return DLL_FAIL;
  }

  if (DLL_SUCCESS != TC4GetFnSeqHeaderExtension(Header.HeaderExtensionVersion, &HeaderExtension, &TechnologyParams))
  {
    cout << "result TC4GetFnSeqHeaderExtension: " << DLL_FAIL << endl;
    return DLL_FAIL;
  }

  int error = 0;
  if (DLL_SUCCESS != TC4CreateSASIrradianceAndTemperatureLookup(&TechnologyParams, Header.HeaderExtensionVersion, &HeaderExtension, &error))
  {
    cout << "result TC4CreateSASIrradianceAndTemperatureLookup: " << DLL_FAIL << endl;
    return DLL_FAIL;
  }

  return DLL_SUCCESS;
}

int port;
double systemVoltagePerInc, systemCurrentPerInc, systemPowerPerInc;
double vphys, iphys, pphys, rphys;
unsigned int majorMinor, build;
Profiler profiler;
char VersionString[DLL_VERSIONSTRING_COPY_LENGTH] = {0};
int sequeceNumber;
int nextIrradianceLevel;
int rampTime;

int main()
{
  DLL_RET_T result = DLL_SUCCESS;

  //setup windows console
  HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
  COORD coord;
  coord.X = 80;
  coord.Y = 10000;
  SetConsoleScreenBufferSize(hConsole, coord);

  // show tcio lib info at start
  result = DllReadVersion(&majorMinor, &build, VersionString);
  if (DLL_SUCCESS != result)
  {
      cout << "reslut DllReadVersion: " << result << endl;
  }
  else
  {
    cout << "TCIO version: V" << (majorMinor >> 0x10 ) << "." << (majorMinor & 0xffff ) << "." << build << endl;
    cout << "TCIO version string: " << VersionString << endl << endl;
  }

  // initialise lib
  result = DllInit();
  if (DLL_SUCCESS != result)
  {
    cout << "reslut DllInit = " << result << endl;
  }

  // manatory call
  if (DLL_SUCCESS == result)
  {
    // connect device (system or module)
    result = DllSearchDevice(1, 8, &port);
    if (DLL_SUCCESS != result)
    {
      cout << "reslut DllSearchDevice = " << result << endl;
    }
    else
    {
      cout << "TopCon device connected to COM port: " << port << endl << endl;
    }
  }

  // manatory call
  if (DLL_SUCCESS == result)
  {
    double dummy = 0.0;
    result = TC4GetPhysicalValuesIncrement(&dummy, &dummy, &dummy, &dummy, &systemVoltagePerInc, &systemCurrentPerInc, &systemPowerPerInc, &dummy);
    if (DLL_SUCCESS != result)
    {
      cout << "result TC4GetPhysicalValuesIncrement: " << result << endl;
    }
  }

  // manatory call
  if (DLL_SUCCESS == result)
  {
    //select system values
    result = TC4SetModuleSelector(64);
    if (DLL_SUCCESS != result)
    {
      cout << "result TC4SetModuleSelector: " << result << endl;
    }
  }

  // manatory SAS call
  if (DLL_SUCCESS == result)
  {
    result = TC4SetFnSeqMaxUserCompatibleVersion(3);
    cout << "result TC4SetFnSeqMaxUserCompatibleVersion: " << result << endl;
  }

  // fetch control to RS232/optional interface
  if (DLL_SUCCESS == result)
  {
    result = TC4SetRemoteControlInput(2);
    if (DLL_SUCCESS != result)
    {
      cout << "result TC4SetRemoteControlInput to RS232: " << result << endl << endl;
    }
  }

  // now operate if init, connect and madatory calls success
  if (DLL_SUCCESS == result)
  {
    // get device serial number
    char serialNr[20] = {0};
    result = TC4GetSerialNrString(20, serialNr);
    if (DLL_SUCCESS != result)
    {
      cout << "result TC4GetSerialNrString: " << result  << endl;
    }
    else
    {
      cout << "Serial number: " << serialNr << endl << endl;;
    }

   // get system physical values
    result = TC4GetSystemPhysicalLimitMax(&vphys, &iphys, &pphys, &rphys);

    if (DLL_SUCCESS != result)
    {
      cout << "result TC4GetSystemPhysicalLimitMax: " << result <<endl;
    }
    else
    {
      cout << "System max voltage: " << vphys << " V" << endl;
      cout << "System max current Q1: " << iphys << " A" << endl;
      cout << "System max Power Q1: " << pphys << " kW" << endl;
      cout << "System max resistance: " << rphys << " mOhm" << endl;
    }

    // get system physical values
    result = TC4GetSystemPhysicalLimitMin(&vphys, &iphys, &pphys, &rphys);

    if (DLL_SUCCESS != result)
    {
      cout << "result TC4GetSystemPhysicalLimitMin: " << result <<endl;
    }
    else
    {
      cout << "System min voltage: " << vphys << " V" << endl;
      cout << "System min current Q1: " << iphys << " A" << endl;
      cout << "System min Power Q1: " << pphys << " kW" << endl;
      cout << "System min resistance: " << rphys << " mOhm" << endl << endl;
    }

    // check set voltage value duration 100 times
    if (DLL_SUCCESS == result)
    {
      profiler.Start();
      for (int i = 0; i < 100; i++)
      {
        result = TC4SetVoltageRef(i);
        if (DLL_SUCCESS != result)
        {
          cout << "result TC4SetVoltageRef: " << result << endl;
          break;
        }
      }
      if (DLL_SUCCESS == result)
      {
        cout << "call 100 times TC4SetVoltageRef duration: " << profiler.GetDiff_ms() << " ms" << endl << endl;
      }
    }

    // get user input
    if (DLL_SUCCESS == result)
    {
      sequeceNumber = GetUserInputInt("Enter FuncGen sequence number to load: ", 1, 1000);
    }

    // load func gen
    if (DLL_SUCCESS == result)
    {
      result = LoadFuncGenSAS(sequeceNumber);
      if (DLL_SUCCESS != result)
      {
        cout << "result LoadFuncGen: " << result << endl;
      }
      else
      {
        cout << "loaded FuncGen sequence number: " << sequeceNumber << endl;
      }
    }

   // get user input
    if (DLL_SUCCESS == result)
    {
      nextIrradianceLevel = GetUserInputInt("Enter next irradiance level [W/m2]: ", 0, 2000);
      rampTime = GetUserInputInt("Enter irradiance ramp time [s]: ", 3, 100);
    }

    // power on
    /*if (DLL_SUCCESS == result)
    {
      cout << endl <<"ATTENTION!!!" << endl << "TopCon now POWER ON to ramp irradiance." << endl;
      PressEnterToContinue();
      cout << endl;

      result = TC4SetControlIn(1);
      if (DLL_SUCCESS != result)
      {
        cout << "result TC4SetControlIn(1): " << result << endl;
      }
    }*/

    // start ramp
    if (DLL_SUCCESS == result)
    {
      // wait after power on
      Sleep(1000);
      result = TC4SetSASIrradianceRef(nextIrradianceLevel, rampTime);

      double irradianceAct, ramptimeAct;
      for (int i = 0; i < rampTime; i++)
      {
        Sleep(1000);
        TC4GetSASIrradianceAct(&irradianceAct, &ramptimeAct);
        cout << "irradianceAct: " << irradianceAct << "W/m2; ramptimeAct: " << ramptimeAct << " s" << endl;
      }
    }

    // power off if connected
    // for fail save power off, use also the watchdog if connection lost, see manual
    if (0 < port)
    {
      bool powerOn = true;
      do{

        //read and clear last communication error if has one...
        int state, errorNo;
        DllGetStatus(&state, &errorNo);

        //now try power off
        result = TC4SetControlIn(0);
        if (DLL_SUCCESS != result)
        {
          cout << "result TC4SetControlIn(0): " << result << endl;
        }
        else
        {
          powerOn = false;
        }
      }
        while(powerOn);
    }
  }

  // always unload tcio lib
  result = DllClose();
  if (DLL_SUCCESS != result)
  {
    cout << "DllClose = " << result << endl;
  }

  //prevent console window closing
  PressEnterToContinue();

  return 0;
}