#include "stdafx.h"
#include <windows.h>

#include <cstdio>
#include <sstream>
#include <iostream>
#include <limits>
#include <string>
#include <ctime>

#include "serialiolib.h"
#include "profiler.h"

using namespace std;




void PressEnterToContinue()
{
  std::cout << "Press ENTER to continue... " << flush;
  std::cin.ignore( std::numeric_limits <std::streamsize> ::max(), '\n' );
}

int port;
double systemVoltagePerInc, systemCurrentPerInc, systemPowerPerInc;
double vphys, iphys, pphys, rphys;
unsigned int majorMinor, build;
Profiler profiler;
char VersionString[DLL_VERSIONSTRING_COPY_LENGTH] = {0};

int main()
{
  DLL_RET_T result = DLL_SUCCESS;

  //setup windows console
  HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
  COORD coord;
  coord.X = 80;
  coord.Y = 10000;
  SetConsoleScreenBufferSize(hConsole, coord);

  // show tcio lib info at start
  result = DllReadVersion(&majorMinor, &build, VersionString);
  if (DLL_SUCCESS != result)
  {
      cout << "reslut DllReadVersion: " << result << endl;
  }
  else
  {
    cout << "TCIO version: V" << (majorMinor >> 0x10 ) << "." << (majorMinor & 0xffff ) << "." << build << endl;
    cout << "TCIO version string: " << VersionString << endl << endl;
  }

  // initialise lib
  result = DllInit();
  if (DLL_SUCCESS != result)
  {
    cout << "reslut DllInit = " << result << endl;
  }

  // manatory call
  if (DLL_SUCCESS == result)
  {
    // connect device (system or module)
    result = DllSearchDevice(1, 8, &port);
    if (DLL_SUCCESS != result)
    {
      cout << "reslut DllSearchDevice = " << result << endl;
    }
    else
    {
      cout << "TopCon device connected to COM port: " << port << endl << endl;
    }
  }

  // manatory call
  if (DLL_SUCCESS == result)
  {
    double dummy = 0.0;
    result = TC4GetPhysicalValuesIncrement(&dummy, &dummy, &dummy, &dummy, &systemVoltagePerInc, &systemCurrentPerInc, &systemPowerPerInc, &dummy);
    if (DLL_SUCCESS != result)
    {
      cout << "result TC4GetPhysicalValuesIncrement: " << result << endl;
    }
  }

  // manatory call
  if (DLL_SUCCESS == result)
  {
    //select system values
    result = TC4SetModuleSelector(64);
    if (DLL_SUCCESS != result)
    {
      cout << "result TC4SetModuleSelector: " << result << endl;
    }
  }

  // fetch control to RS232/optional interface
  if (DLL_SUCCESS == result)
  {
    result = TC4SetRemoteControlInput(2);
    if (DLL_SUCCESS != result)
    {
      cout << "result TC4SetRemoteControlInput to RS232: " << result << endl << endl;
    }
  }

  // now operate if init, connect and madatory calls success
  if (DLL_SUCCESS == result)
  {
    // get device serial number
    char serialNr[20] = {0};
    result = TC4GetSerialNrString(20, serialNr);
    if (DLL_SUCCESS != result)
    {
      cout << "result TC4GetSerialNrString: " << result  << endl;
    }
    else
    {
      cout << "Serial number: " << serialNr << endl << endl;;
    }

   // get system physical values
    result = TC4GetSystemPhysicalLimitMax(&vphys, &iphys, &pphys, &rphys);

    if (DLL_SUCCESS != result)
    {
      cout << "result TC4GetSystemPhysicalLimitMax: " << result <<endl;
    }
    else
    {
      cout << "System max voltage: " << vphys << " V" << endl;
      cout << "System max current Q1: " << iphys << " A" << endl;
      cout << "System max Power Q1: " << pphys << " kW" << endl;
      cout << "System max resistance: " << rphys << " mOhm" << endl;
    }

    // get system physical values
    result = TC4GetSystemPhysicalLimitMin(&vphys, &iphys, &pphys, &rphys);

    if (DLL_SUCCESS != result)
    {
      cout << "result TC4GetSystemPhysicalLimitMin: " << result <<endl;
    }
    else
    {
      cout << "System min voltage: " << vphys << " V" << endl;
      cout << "System min current Q1: " << iphys << " A" << endl;
      cout << "System min Power Q1: " << pphys << " kW" << endl;
      cout << "System min resistance: " << rphys << " mOhm" << endl << endl;
    }

    // check set voltage value duration 100 times
    if (DLL_SUCCESS == result)
    {
      profiler.Start();
      for (int i = 0; i < 100; i++)
      {
        result = TC4SetVoltageRef(i);
        if (DLL_SUCCESS != result)
        {
          cout << "result TC4SetVoltageRef: " << result << endl;
          break;
        }
      }
      if (DLL_SUCCESS == result)
      {
        cout << "call 100 times TC4SetVoltageRef duration: " << profiler.GetDiff_ms() << " ms" << endl << endl;
      }
    }

    //
    //exend here your sample code ...
    //

  }

  // always unload tcio lib
  result = DllClose();
  if (DLL_SUCCESS != result)
  {
    cout << "DllClose = " << result << endl;
  }
  PressEnterToContinue();

  return 0;
}